from django_kafka.utils.message import Header


class DeadLetterHeader(Header):
    MESSAGE_TOPIC = "DEAD_LETTER_MESSAGE_TOPIC"
    MESSAGE_PARTITION = "DEAD_LETTER_MESSAGE_PARTITION"
    MESSAGE_OFFSET = "DEAD_LETTER_MESSAGE_OFFSET"
    SUMMARY = "DEAD_LETTER_SUMMARY"
    DETAIL = "DEAD_LETTER_DETAIL"
