# Generated by Django 5.2 on 2025-07-03 12:27

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('django_kafka', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='WaitingMessage',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('status', models.IntegerField(choices=[(1, 'Waiting'), (2, 'Resolving'), (3, 'Resolved')], default=1, verbose_name='status')),
                ('key', models.BinaryField(verbose_name='key')),
                ('value', models.BinaryField(verbose_name='value')),
                ('timestamp', models.JSONField(verbose_name='kafka timestamp')),
                ('topic', models.TextField(verbose_name='topic')),
                ('partition', models.TextField(verbose_name='partition')),
                ('offset', models.TextField(verbose_name='offset')),
                ('_headers', models.JSONField(null=True, verbose_name='headers')),
                ('relation_model_key', models.CharField(max_length=255, verbose_name='relation model key')),
                ('relation_id_field', models.CharField(max_length=256, verbose_name='relation id field')),
                ('relation_id_value', models.CharField(max_length=256, verbose_name='relation id value')),
                ('serialized_relation', models.JSONField(verbose_name='relation kwargs')),
            ],
            options={
                'verbose_name': 'waiting message',
                'verbose_name_plural': 'waiting messages',
                'ordering': ('offset',),
            },
        ),
    ]
