# Generated by Django 5.1.6 on 2025-03-31 14:06

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='KeyOffsetTracker',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('topic', models.CharField(max_length=256, verbose_name='topic')),
                ('key', models.BinaryField(verbose_name='key')),
                ('offset', models.PositiveIntegerField(verbose_name='offset')),
                ('create_time', models.DateTimeField(null=True, verbose_name='create time')),
                ('log_append_time', models.DateTimeField(null=True, verbose_name='log append time')),
            ],
            options={
                'verbose_name': 'key offset tracker',
                'verbose_name_plural': 'key offsets tracker',
                'indexes': [models.Index(fields=['topic', 'key'], include=('offset',), name='django_kafka_key_offset_index')],
            },
        ),
    ]
